#Import-LocalizedData.ps1
# Auteur Jacques Barathon alias Janel. Le 20/01/2009

function Import-LocalizedData([string]$FileName = [IO.Path]::GetFileNameWithoutExtension((split-path -Leaf $MyInvocation.MyCommand)),
                              [System.Globalization.CultureInfo]$UIculture = (Get-UIculture))

{
# La fonction ci-dessus permet d'importer des donnes selon la culture de la session en cours ou en prcisant une culture particulire.
# Evidemment (c'tait le but de l'exercice) a marche avec la v1 de PowerShell. 
# 
# Chaque fichier de donnes doit tre plac dans un rpertoire portant le nom de la culture ("fr-FR", "en-US", etc). 
# Ce rpertoire lui-mme doit tre plac dans le mme rpertoire que le script. 
# 
# Par dfaut, le fichier lu aura le mme nom que le script, avec une extension diffrente: .psl1 (pour PowerShell Localization, v1 ).
# Mais on peut prciser un autre nom de fichier. Attention  ne pas prciser l'extension, elle est rajoute par la fonction! 
# 
# Enfin, le contenu d'un fichier doit tre sous la forme suivante : 
# <nom> = <valeur> 
# 
# Pour faciliter la lecture du fichier, on peut ajouter des lignes vierges et des lignes de commentaire commenant par #.
#
# get-content c:\tests\fr-FR\test-local.psl1
# #culture "fr-FR"
# 
# Welcome = Bienvenue au jeu "Devinez un jour" !
# Prompt  = Tapez un nom de jour
# Success = Vous avez devin ! Toutes nos flicitations !
# 
# get-content c:\tests\en-US\test-local.psl1
# # culture "en-US"
# Welcome = Welcome to the "Guess a day" game!
# Prompt = Enter the name of any day
# Success = You had it right! Congratulations!

  $FullPath = $(split-path $myinvocation.ScriptName) + "\$UIculture\$FileName.psl1"

  $ht = @{}
  Get-Content $FullPath |`
    Where {($_.TrimStart()) -ne [String]::Empty -and ($_.TrimStart())[0] -ne "#"} |`
    Foreach { $key, $value = $_.Split("=", 2)
              $ht.Add($key.Trim(), $value.TrimStart())
    }
  $ht
}

# SIG # Begin signature block
# MIIFQgYJKoZIhvcNAQcCoIIFMzCCBS8CAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQU1V+FT0PBoHWps8PxBkdz4Hu8
# WiqgggL8MIIC+DCCAmWgAwIBAgIQGpdOSYj2EbRAxGqmgFirkTAJBgUrDgMCHQUA
# MHsxeTB3BgNVBAMecABMAGEAdQByAGUAbgB0ACAARABhAHIAZABlAG4AbgBlACAA
# YQB1AHQAbwByAGkAdADpACAAZABlACAAYwBlAHIAdABpAGYAaQBjAGEAdABpAG8A
# bgAgAHIAYQBjAGkAbgBlACAAbABvAGMAYQBsAGUwHhcNMDcwNzAxMTMyNjU5WhcN
# MzkxMjMxMjM1OTU5WjA2MTQwMgYDVQQDEytMYXVyZW50IERhcmRlbm5lIGNlcnRp
# ZmljYXQgcG91ciBQb3dlclNoZWxsMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKB
# gQCfu+w77PdPXH8+C41SaR48k/DPql1EPDL8O4gArRwbKH/McvXmXCKEwaWYlFi7
# w8F4CwO/kBswENa+0X3OXLUOK0LyQmJP7VQqA+mT4Up+a5Z3mcsRd1Out+OzfOuR
# hium1zhE1/MlIqEK6hnMl/A/bkc4SCFfdiJeZc83tNkGXwIDAQABo4HJMIHGMBMG
# A1UdJQQMMAoGCCsGAQUFBwMDMIGuBgNVHQEEgaYwgaOAEB0w3fWBsN0e2nTMjcGF
# f8qhfTB7MXkwdwYDVQQDHnAATABhAHUAcgBlAG4AdAAgAEQAYQByAGQAZQBuAG4A
# ZQAgAGEAdQB0AG8AcgBpAHQA6QAgAGQAZQAgAGMAZQByAHQAaQBmAGkAYwBhAHQA
# aQBvAG4AIAByAGEAYwBpAG4AZQAgAGwAbwBjAGEAbABlghAb5Gp5W/j2oEZ4E3Mn
# LmpOMAkGBSsOAwIdBQADgYEAoEjrLmRoEscvqLGp6RXFH55NjCul7e118oWxlpHt
# hcme2FZVN0vNB0Xqa+A3YU4QyYhYNeBaJ/gsgv1MC7PnuBR2ek58mTwVa6WlVNrn
# KK8A7P3MRVCOGVYkOiw5xttWFvPPph1YG1CAAwAwSI+nIfJCxyJDceOvvbCoV+US
# FLgxggGwMIIBrAIBATCBjzB7MXkwdwYDVQQDHnAATABhAHUAcgBlAG4AdAAgAEQA
# YQByAGQAZQBuAG4AZQAgAGEAdQB0AG8AcgBpAHQA6QAgAGQAZQAgAGMAZQByAHQA
# aQBmAGkAYwBhAHQAaQBvAG4AIAByAGEAYwBpAG4AZQAgAGwAbwBjAGEAbABlAhAa
# l05JiPYRtEDEaqaAWKuRMAkGBSsOAwIaBQCgeDAYBgorBgEEAYI3AgEMMQowCKAC
# gAChAoAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsx
# DjAMBgorBgEEAYI3AgEVMCMGCSqGSIb3DQEJBDEWBBSDi0yOet+1aMPvwTqpurQ1
# uwbuAzANBgkqhkiG9w0BAQEFAASBgGX7qgbjS1NfVdZaCUDtOxdzFzIX1sksxmwO
# KD6sNmPo4UZ4OOMFr9J/rsiS6lswR9yvwLfY3qnVaQaCdPVTUHut25d0r49jP+i2
# PioOEUv/7aGpSb8Sg4ul31zXdBMF0Xtj6Xv96VaFzjurjP1spjssD6HmPUQaXzNM
# LB/84zwa
# SIG # End signature block
